@@ -5,6 +5,7 @@ CHUK MCP Server
 * automatic session-ID injection for artifact tools via native session management
 * optional bearer-token auth middleware
 * transparent chuk_artifacts integration
+* MCP resources integration with session-isolated artifact access
 * global **and per-tool** timeout support
 * end-to-end **token streaming** for async-generator tools
 * JSON concatenation fixing for tool parameters
@@ -15,6 +16,7 @@ from __future__ import annotations
 import asyncio
 import contextlib
 import importlib
+import inspect
 import json
 import os
 import re
@@ -30,12 +32,18 @@ from typing import Any, AsyncIterator, Callable, Dict, List, Optional, Union
 import uvicorn
 
 # ─────────────────────────── chuk_artifacts integration ──────────────────────
-from chuk_artifacts import ArtifactStore
+from chuk_artifacts import ArtifactNotFoundError, ArtifactStore
 from mcp.server import Server
 from mcp.server.sse import SseServerTransport
 from mcp.server.stdio import stdio_server
 from mcp.server.streamable_http_manager import StreamableHTTPSessionManager
-from mcp.types import EmbeddedResource, ImageContent, TextContent, Tool
+from mcp.types import (
+    EmbeddedResource,
+    ImageContent,
+    Resource,
+    TextContent,
+    Tool,
+)
 from starlette.applications import Starlette
 from starlette.exceptions import HTTPException
 from starlette.middleware import Middleware
@@ -44,6 +52,11 @@ from starlette.responses import JSONResponse, PlainTextResponse, Response
 from starlette.routing import Mount, Route
 from starlette.types import ASGIApp, Receive, Scope, Send
 
+from chuk_mcp_runtime.common.mcp_resource_decorator import (
+    get_registered_resources,
+    get_resource_function,
+    initialize_resource_registry,
+)
 from chuk_mcp_runtime.common.mcp_tool_decorator import (
     TOOLS_REGISTRY,
     initialize_tool_registry,
@@ -55,6 +68,7 @@ from chuk_mcp_runtime.server.logging_config import get_logger
 from chuk_mcp_runtime.server.request_context import (
     MCPRequestContext,
     set_request_context,
+    set_request_headers,
 )
 from chuk_mcp_runtime.session.native_session_management import (
     MCPSessionManager,
@@ -152,13 +166,16 @@ class AuthMiddleware:
         path: str = scope["path"]
         method: str = scope["method"]
 
-        # Health check or “auth disabled” → no checks.
+        # --------------------  Extract headers  --------------------
+        headers = {k.decode().lower(): v.decode() for k, v in scope["headers"]}
+        scope["headers_dict"] = headers  # Always store headers for request context
+
+        # Health check or "auth disabled" → no checks.
         if (path == self.health_path and method == "GET") or self.auth != "bearer":
             await self.app(scope, receive, send)
             return
 
         # --------------------  Extract token  --------------------
-        headers = {k.decode().lower(): v.decode() for k, v in scope["headers"]}
         token: Optional[str] = None
 
         # 1)  Authorization: Bearer <token>
@@ -348,6 +365,7 @@ class MCPServer:
             self.tools_registry = await self._import_tools_registry()
 
         await initialize_tool_registry()
+        await initialize_resource_registry()
         update_naming_maps()
 
         server = Server(self.server_name)
@@ -357,7 +375,7 @@ class MCPServer:
         async def list_tools() -> List[Tool]:
             """List available tools with robust error handling."""
             try:
-                self.logger.info("list_tools called - %d tools total", len(self.tools_registry))
+                self.logger.debug("list_tools called - %d tools total", len(self.tools_registry))
 
                 tools = []
                 for tool_name, func in self.tools_registry.items():
@@ -382,7 +400,7 @@ class MCPServer:
                         self.logger.error("Error processing tool %s: %s", tool_name, e)
                         continue
 
-                self.logger.info("Returning %d valid tools", len(tools))
+                self.logger.debug("Returning %d valid tools", len(tools))
                 return tools
 
             except Exception as e:
@@ -403,7 +421,7 @@ class MCPServer:
                         arguments = parse_tool_arguments(arguments)
 
                         if arguments != original_args:
-                            self.logger.info(
+                            self.logger.debug(
                                 "Fixed concatenated JSON arguments for '%s': %s -> %s",
                                 name,
                                 original_args,
@@ -454,11 +472,20 @@ class MCPServer:
                 arguments = await self._inject_session_context(resolved, arguments)
 
                 # Execute within session context
+                # Use MCP session if available, otherwise use session_id from arguments
+                effective_session_id = (
+                    mcp_session.session_id
+                    if mcp_session and hasattr(mcp_session, "session_id")
+                    else arguments.get("session_id")
+                )
                 async with SessionContext(
                     self.session_manager,
-                    session_id=arguments.get("session_id"),
+                    session_id=effective_session_id,
                     auto_create=True,
                 ) as session_id:
+                    self.logger.info(
+                        f"[SESSION] Executing tool '{resolved}' in session {session_id}"
+                    )
                     self.logger.debug(
                         "Executing tool '%s' in session %s (progress_token=%s)",
                         resolved,
@@ -558,6 +585,159 @@ class MCPServer:
                 self.logger.error("Error in call_tool for '%s': %s", name, e)
                 return [TextContent(type="text", text=f"Tool execution error: {str(e)}")]
 
+        # ----------------------------- list_resources ----------------------------- #
+        @server.list_resources()
+        async def list_resources() -> List[Resource]:
+            """
+            List all available resources:
+            1. Custom resources (from @mcp_resource decorators)
+            2. Artifact resources (session-isolated files)
+            """
+            all_resources = []
+
+            # Part 1: Add custom resources (from decorators)
+            try:
+                custom_resources = get_registered_resources()
+                all_resources.extend(custom_resources)
+                self.logger.debug("Added %d custom resources", len(custom_resources))
+            except Exception as e:
+                self.logger.error("Failed to list custom resources: %s", e)
+
+            # Part 2: Add artifact resources (session-isolated)
+            artifacts_config = self.config.get("artifacts", {})
+            if artifacts_config.get("enabled", False):
+                current_session = self.session_manager.get_current_session()
+
+                if current_session and self.artifact_store:
+                    try:
+                        # CRITICAL: Only list files in the CURRENT session
+                        files = await self.artifact_store.list_by_session(current_session)
+
+                        for f in files:
+                            artifact_id = f.get("artifact_id", "")
+                            if not artifact_id:
+                                continue
+
+                            uri = f"artifact://{artifact_id}"
+
+                            all_resources.append(
+                                Resource(
+                                    uri=uri,
+                                    name=f.get("filename", "unknown"),
+                                    description=f.get("summary", ""),
+                                    mimeType=f.get("mime", "application/octet-stream"),
+                                )
+                            )
+
+                        self.logger.debug(
+                            "Added %d artifact resources for session %s",
+                            len(files),
+                            current_session,
+                        )
+                    except Exception as e:
+                        self.logger.error("Failed to list artifact resources: %s", e)
+                else:
+                    if not current_session:
+                        self.logger.debug("No session context - skipping artifact resources")
+                    if not self.artifact_store:
+                        self.logger.debug("Artifact store not initialized")
+
+            self.logger.debug("Listing %d total resources", len(all_resources))
+            return all_resources
+
+        # ----------------------------- read_resource ----------------------------- #
+        @server.read_resource()
+        async def read_resource(uri: str):
+            """
+            Read resource content from custom resources or artifacts.
+
+            Handles:
+            1. Custom resources (from @mcp_resource decorators)
+            2. Artifact resources (session-isolated files)
+            """
+
+            # Try custom resources first
+            # Convert AnyUrl to string if needed
+            uri_str = str(uri)
+            resource_func = get_resource_function(uri_str)
+            if resource_func:
+                try:
+                    self.logger.debug("Reading custom resource: %s", uri)
+
+                    # Call the resource function
+                    result = resource_func()
+                    if inspect.iscoroutine(result):
+                        result = await result
+
+                    # Get MIME type from metadata
+                    mime_type = getattr(resource_func, "_mcp_resource", None)
+                    mime = mime_type.mimeType if mime_type else "text/plain"
+
+                    # SDK accepts: str | bytes | Iterable[ReadResourceContents]
+                    # Return raw string/bytes for simple cases
+                    if isinstance(result, bytes):
+                        return result
+                    else:
+                        return str(result)
+
+                except Exception as e:
+                    self.logger.error("Failed to read custom resource %s: %s", uri, e)
+                    raise ValueError(f"Failed to read custom resource: {str(e)}")
+
+            # Try artifact resources
+            if uri_str.startswith("artifact://"):
+                artifacts_config = self.config.get("artifacts", {})
+                if not artifacts_config.get("enabled", False):
+                    raise ValueError("Artifacts not enabled")
+
+                current_session = self.session_manager.get_current_session()
+                if not current_session:
+                    raise ValueError("No session context - cannot read artifact resource")
+
+                if not self.artifact_store:
+                    raise ValueError("Artifact store not initialized")
+
+                try:
+                    artifact_id = uri.replace("artifact://", "").strip("/")
+                    if not artifact_id:
+                        raise ValueError("Invalid URI: missing artifact ID")
+
+                    # Get metadata first to verify session ownership
+                    metadata = await self.artifact_store.metadata(artifact_id)
+                    resource_session = metadata.get("session_id")
+
+                    # CRITICAL SECURITY CHECK: Verify session ownership
+                    if resource_session != current_session:
+                        self.logger.warning(
+                            "Session isolation violation: artifact %s belongs to %s, "
+                            "requested by %s",
+                            artifact_id,
+                            resource_session,
+                            current_session,
+                        )
+                        raise ValueError("Access denied: resource belongs to different session")
+
+                    # Session validated - safe to read
+                    data = await self.artifact_store.retrieve(artifact_id)
+                    mime = metadata.get("mime", "application/octet-stream")
+
+                    # SDK accepts: str | bytes | Iterable[ReadResourceContents]
+                    # Return raw string/bytes for simple cases
+                    if mime.startswith("text/"):
+                        return data.decode("utf-8")
+                    else:
+                        return data
+
+                except ArtifactNotFoundError:
+                    self.logger.warning("Artifact resource not found: %s", uri)
+                    raise ValueError(f"Resource not found: {uri}")
+                except Exception as e:
+                    self.logger.error("Failed to read artifact resource %s: %s", uri, e)
+                    raise ValueError(f"Failed to read resource: {str(e)}")
+
+            # Resource not found
+            raise ValueError(f"Resource not found: {uri}")
+
         # ------------------------------------------------------------------ #
         # Transport bootstrapping (stdio / SSE / StreamableHTTP)            #
         # ------------------------------------------------------------------ #
@@ -580,6 +760,10 @@ class MCPServer:
             transport = SseServerTransport(msg_path)
 
             async def _handle_sse(request: Request):
+                # Extract and store headers for tools to access
+                headers_dict = request.scope.get("headers_dict", {})
+                set_request_headers(headers_dict)
+
                 async with transport.connect_sse(
                     request.scope, request.receive, request._send
                 ) as streams:
@@ -637,6 +821,10 @@ class MCPServer:
             async def handle_streamable_http(
                 scope: Scope, receive: Receive, send: Send
             ) -> Response:
+                # Extract and store headers for tools to access
+                headers_dict = scope.get("headers_dict", {})
+                set_request_headers(headers_dict)
+
                 await session_manager.handle_request(scope, receive, send)
                 return Response()
 
